# chosung.py

from hangulpy.utils import CHOSUNG_LIST, CHOSUNG_BASE, is_hangul_char, HANGUL_BEGIN_UNICODE

def get_chosung(c):
    """
    주어진 한글 음절의 초성을 반환합니다.
    
    :param c: 한글 음절 문자
    :return: 초성 문자 (한글이 아니면 그대로 반환)
    """
    if is_hangul_char(c):
        char_index = ord(c) - HANGUL_BEGIN_UNICODE
        chosung_index = char_index // CHOSUNG_BASE
        return CHOSUNG_LIST[chosung_index]
    return c

def chosungIncludes(word, pattern):
    """
    주어진 단어에 패턴의 초성이 포함되어 있는지 확인합니다.
    
    :param word: 검색할 단어
    :param pattern: 검색할 초성 패턴
    :return: 패턴이 단어의 초성에 포함되어 있으면 True, 아니면 False
    """
    # 단어의 각 문자를 초성으로 변환하여 문자열을 생성합니다.
    word_chosung = ''.join(get_chosung(c) for c in word)
    # 패턴이 초성 문자열에 포함되어 있는지 확인합니다.
    return pattern in word_chosung



# hangul_role.py

from hangulpy.utils import CHOSUNG_LIST, JONGSUNG_LIST

def can_be_chosung(char):
    """
    주어진 문자가 한글 초성으로 쓰일 수 있는지 확인합니다.
    
    :param char: 확인할 문자
    :return: 초성으로 쓰일 수 있으면 True, 아니면 False
    """
    return char in CHOSUNG_LIST

def can_be_jongsung(char):
    """
    주어진 문자가 한글 종성으로 쓰일 수 있는지 확인합니다.
    
    :param char: 확인할 문자
    :return: 종성으로 쓰일 수 있으면 True, 아니면 False
    """
    return char in JONGSUNG_LIST



# hangul_contains.py

from hangulpy.utils import is_hangul_char
from hangulpy.hangul_split import split_hangul_char

def hangul_contains(word, pattern, notallowempty=False):
    """
    주어진 한글 문자열이 다른 한글 문자열을 포함하는지 검사합니다.
    
    :param word: 검사할 한글 문자열
    :param pattern: 포함 여부를 검사할 한글 문자열 패턴
    :param notallowempty: 패턴이 빈 문자열일 때 false를 반환하는 옵션
    :return: 포함되면 True, 아니면 False
    """
    if not pattern:
        return not notallowempty
    
    # 문자열을 분해하여 리스트로 변환
    word_split = ''.join(''.join(split_hangul_char(char)) for char in word)
    pattern_split = ''.join(''.join(split_hangul_char(char)) for char in pattern)
    
    return pattern_split in word_split



from setuptools import setup, find_packages

setup(
    name='hangulpy',
    version='1.0.1',
    description='A Python library for processing Hangul, inspired by es-hangul.',
    long_description=open('README.md', encoding='utf-8').read(),
    long_description_content_type='text/markdown',
    url='https://github.com/gaon12/hangulpy',
    author='Jeong Gaon',
    author_email='gokirito12@gmail.com',
    license='MIT',
    packages=find_packages(),
    install_requires=[],
    classifiers=[
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
    ],
    python_requires='>=3.8',
)



# hangul_decompose.py

from hangulpy.utils import CHOSUNG_LIST, CHOSUNG_BASE, is_hangul_char, HANGUL_BEGIN_UNICODE, JUNGSUNG_LIST, JUNGSUNG_DECOMPOSE, JONGSUNG_LIST, JONGSUNG_DECOMPOSE, JUNGSUNG_BASE

def decompose_hangul_char(c):
    """
    주어진 한글 음절을 초성, 중성, 종성으로 분해합니다.
    
    :param c: 한글 음절 문자
    :return: (초성, 중성, 종성) 튜플
    """
    if is_hangul_char(c):
        # 한글 음절의 유니코드 값을 기준으로 각 성분의 인덱스를 계산합니다.
        char_index = ord(c) - HANGUL_BEGIN_UNICODE
        chosung_index = char_index // CHOSUNG_BASE
        jungsung_index = (char_index % CHOSUNG_BASE) // JUNGSUNG_BASE
        jongsung_index = char_index % JUNGSUNG_BASE
        
        # 중성 및 종성 분해
        jungsung = JUNGSUNG_LIST[jungsung_index]
        jongsung = JONGSUNG_LIST[jongsung_index]

        jungsung_decomposed = JUNGSUNG_DECOMPOSE.get(jungsung, (jungsung,))
        jongsung_decomposed = JONGSUNG_DECOMPOSE.get(jongsung, (jongsung,))
        
        return (CHOSUNG_LIST[chosung_index], jungsung_decomposed, jongsung_decomposed)
    
    return (c, '', '')  # 한글이 아니면 그대로 반환
